# Copyright (C) 2016 Canonical Ltd.
# Copyright (C) 2016 VMware Inc.
#
# Author: Sankar Tanguturi <stanguturi@vmware.com>
#
# This file is part of cloud-init. See LICENSE file for license information.


class GuestCustStateEnum(object):
    """Specifies different states of Guest Customization engine"""

    GUESTCUST_STATE_RUNNING = 4
    GUESTCUST_STATE_DONE = 5

# vi: ts=4 expandtab
